ACC <- function(var_exp, var_obs, lon = NULL, lat = NULL, lonlatbox = NULL, conf=TRUE) {
  # Matrix var_exp & var_obs should have dimensions (nexp/nobs, nsdates, 
  # nltimes, nlat, nlon).
  # ACC computes the Anomaly Correlation Coefficient for each jexp in 
  # 1:nexp and each jobs in 1:nobs which gives nexp x nobs ACC for each 
  # startdate and each leadtime.
  # A domain can be selected by providing the list of longitudes/latitudes
  # (lon/lat) of the grid together with the corner of the domain:
  #   lonlatbox = c(lonmin, lonmax, latmin, latmax)
  # 
  # Args:
  #   var_exp: Matrix of experimental data.
  #   var_obs: Matrix of observational data, same dimensions as var_exp except 
  #            along the first dimension.
  #   lon: Array of longitudes of the var_exp/var_obs grids, optional.
  #   lat: Array of latitudes of the var_exp/var_obs grids, optional.
  #   lonlatbox: Domain to select c(lonmin, lonmax, latmin, latmax), optional.
  #   conf: TRUE/FALSE if conf equal TRUE:
  #         The confidence interval is computed by a Fisher transformation.
  #         The significance level relies on a one-sided student-T distribution.
  #         If conf equal FALSE: no confidence interval is calculated
  #
  # Returns:
  #   $ACC: if conf equal TRUE:
  #        Matrix with c(nexp, nobs, nsdates, nleadtimes, 4) dimensions.
  #        The fourth dimension of length 4 corresponds to the lower limit of
  #        the 95% confidence interval, the computed ACC, the upper limit of 
  #        the 95% confidence interval and the 95% significance level given by 
  #        a one-sided T-test.
  #        if conf equal FALSE:
  #        ACC with c(nexp, nobs, nleadtimes) dimensions.
  #   $MACC: Mean Anomaly Correlation Coefficient with c(nexp, nobs, nleadtimes) 
  #         dimensions.
  # 
  # History:
  #   1.0  #  2013-08  (V. Guemas, vguemas@ic3.cat)  #  Original code
  
  dimsvar <- dim(var_exp)
  if (length(dimsvar) != 5) {
    stop("var_exp & var_obs should have dimensions (nexp/nsobs, nsdates, nltimes, nlat, nlon) ")
  }
  for (iind in 2:length(dimsvar)) {
    if (dim(var_obs)[iind] != dimsvar[iind]) {
      stop("var_exp & var_obs must have same dimensions except the first one (number of experiments or number of observational datasets) ")
    }
  }
  nexp <- dimsvar[1]
  nobs <- dim(var_obs)[1]
  nsdates <- dimsvar[2]
  nltimes <- dimsvar[3]
  nlat <- dimsvar[4]
  nlon <- dimsvar[5]
  if (is.null(lon) == FALSE & is.null(lat) == FALSE & 
      is.null(lonlatbox) == FALSE) {
    for (jind in 1:2) {
      while (lonlatbox[jind] < 0) {
        lonlatbox[jind] <- lonlatbox[jind] + 360
      }
      while (lonlatbox[jind] > 360) {
        lonlatbox[jind] <- lonlatbox[jind] - 360
      }
    }
    indlon <- which((lon >= lonlatbox[1] & lon <= lonlatbox[2]) | 
                    (lonlatbox[1] > lonlatbox[2] & (lon > lonlatbox[1] | 
                    lon < lonlatbox[2])))
    indlat <- which(lat >= lonlatbox[3] & lat <= lonlatbox[4])
  } else {
    indlon <- 1:nlon
    indlat <- 1:nlat
  }
  ACC <- array(NA,dim = c(nexp, nobs, nsdates, nltimes))
  MACCaux <- array(0, dim = c(nexp, nobs, nltimes,4))

  tmp1 <- array(var_exp, 
                dim = c(nexp, nsdates, length(indlon) * length(indlat)))
  tmp2 <- array(var_obs, 
                dim = c(nobs, nsdates, length(indlon) * length(indlat)))


  toto <- sapply(tmp1,tmp2,c(1,2),function(x,y) cor(x,y, use = "na.or.complete"))
  ACC[, , , , 2] <- toto
  MACCaux[, , , 1]=sapply(tmp1,tmp2,c(1,2),function(x,y) sum(tmp1*tmp2, na.rm = TRUE))
  MACCaux[, , , 2]= + sum(tmp1[notna] ^ 2)
            bottom2 <- bottom2 + sum(tmp2[notna] ^ 2)
            numdates <- numdates + 1
          }
        }
        if (numdates > 0) {
          MACC[jexp, jobs, jltime] <- top / sqrt(bottom1 * bottom2)
        }
      }
    }
  }
  invisible(list(ACC = ACC, MACC = MACC))
}
